/*
 * This file and its contents are supplied under the terms of the
 * Common Development and Distribution License ("CDDL"), version 1.0.
 * You may only use this file in accordance with the terms of version
 * 1.0 of the CDDL.
 *
 * A full copy of the text of the CDDL should have accompanied this
 * source.  A copy of the CDDL is also available via the Internet at
 * http://www.illumos.org/license/CDDL.
 */

/*
 * Copyright 2013 Damian Bogel.  All rights reserved.
 */

#ifndef _FSH_H
#define	_FSH_H

#include <sys/types.h>
#include <sys/vfs.h>
#include <sys/vnode.h>

#ifdef __cplusplus
extern "C" {
#endif

struct fsh_int;
typedef struct fsh_int fsh_int_t;

#define	FSH_OPS								\
/* vnode */								\
int (*hook_read)(fsh_int_t *fsh_int, void *arg, vnode_t *vp,		\
	uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);	\
int (*hook_write)(fsh_int_t *fsh_int, void *arg, vnode_t *vp,		\
	uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);	\
/* vfs */								\
int (*hook_mount)(fsh_int_t *fsh_int, void *arg, vfs_t *vfsp,		\
	vnode_t *mvp, struct mounta *uap, cred_t *cr);			\
int (*hook_unmount)(fsh_int_t *fsh_int, void *arg, vfs_t *vfsp,		\
	int flag, cred_t *cr)						\

/* API */
typedef struct fsh {
	void	*arg;
	FSH_OPS;
} fsh_t;

extern void fsh_hook_install(vfs_t *vfsp, fsh_t *hooks);
extern void fsh_hook_remove(vfs_t *vfsp, fsh_t *hooks);

typedef struct fsh_callback {
	void	*fshc_arg;
	void	(*fshc_free)(vfs_t *vfsp, void *arg);
	void	(*fshc_mount)(vfs_t *vfsp, void *arg);
} fsh_callback_t;

extern void fsh_callback_install(fsh_callback_t *fsh_callback);
extern void fsh_callback_remove(fsh_callback_t *fsh_callback);

extern void fsh_fs_enable(vfs_t *vfsp);
extern void fsh_fs_disable(vfs_t *vfsp);

/* FSH control passing */
extern int fsh_next_read(fsh_int_t *fsh_int, vnode_t *vp,
		uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);
extern int fsh_next_write(fsh_int_t *fsh_int, vnode_t *vp,
		uio_t *uiop, int ioflag, cred_t *cr, caller_context_t *ct);

extern int fsh_next_mount(fsh_int_t *fsh_int, vfs_t *vfsp,
		vnode_t *mvp, struct mounta *uap, cred_t *cr);
extern int fsh_next_unmount(fsh_int_t *fsh_int, vfs_t *vfsp,
		int flag, cred_t *cr);

#ifdef __cplusplus
}
#endif

#endif /* _FSH_H */
